/* Name: main.c
 * Project: AVR MIDI device on Low-Speed USB
 * Author: morecat_lab , Yoshitaka Kuwata
 *
 * 2009/01/12 Created (for ginger-pgs)
 * 2010/08/10 modified for Monaka + PSG shield
 *
 * Based on : Martin Homuth-Rosemann
 * Creation Date: 2008-03-11
 * Copyright: (c) 2008 by Martin Homuth-Rosemann.
 * License: GPL v2 or later.
 *
 */

#include <string.h>
#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/pgmspace.h>
#include <avr/wdt.h>
#include <util/delay.h>

#include "usbdrv.h"
#include "oddebug.h"

/* #define TEST */

extern void parseMidiMessage(uchar *, uchar );

/* ----------------------------------------------- */
/*  define PSG configuratuon */

#ifdef MONAKA_PSG
#define N_PSG 1
#else
#define N_PSG 2
#endif

#if (N_PSG == 1)
#define MAX_NOTE 3
#else
#define MAX_NOTE 6
#endif

/* ----------------------------------------------- */
/* グローバル変数 */

#include "freq_table.c"

#define NOTE_OFF 0xff

struct _note {
  uchar psgNo;
  uchar ch;
  uchar noteNo;
  uchar volume;
  uchar order;
} notes[MAX_NOTE] = {
  {0, 0, NOTE_OFF, 0, 0},
  {0, 1, NOTE_OFF, 0, 0},
  {0, 2, NOTE_OFF, 0, 0},
#if (N_PGS == 2) 
  {1, 0, NOTE_OFF, 0, 0},
  {1, 1, NOTE_OFF, 0, 0},
  {1, 2, NOTE_OFF, 0, 0},
#endif
};

static uchar onCount = 0;

// This deviceDescrMIDI[] is based on 
// http://www.usb.org/developers/devclass_docs/midi10.pdf
// Appendix B. Example: Simple MIDI Adapter (Informative)

// B.1 Device Descriptor
static PROGMEM char deviceDescrMIDI[] = {	/* USB device descriptor */
	18,			/* sizeof(usbDescriptorDevice): length of descriptor in bytes */
	USBDESCR_DEVICE,	/* descriptor type */
	0x10, 0x01,		/* USB version supported */
	0,			/* device class: defined at interface level */
	0,			/* subclass */
	0,			/* protocol */
	8,			/* max packet size */
	USB_CFG_VENDOR_ID,	/* 2 bytes */
	USB_CFG_DEVICE_ID,	/* 2 bytes */
	USB_CFG_DEVICE_VERSION,	/* 2 bytes */
	1,			/* manufacturer string index */
	2,			/* product string index */
	0,			/* serial number string index */
	1,			/* number of configurations */
};

// B.2 Configuration Descriptor
static PROGMEM char configDescrMIDI[] = {	/* USB configuration descriptor */
	9,			/* sizeof(usbDescrConfig): length of descriptor in bytes */
	USBDESCR_CONFIG,	/* descriptor type */
	101, 0,			/* total length of data returned (including inlined descriptors) */
	2,			/* number of interfaces in this configuration */
	1,			/* index of this configuration */
	0,			/* configuration name string index */
#if USB_CFG_IS_SELF_POWERED
	USBATTR_SELFPOWER,	/* attributes */
#else
    0x80, /* USBATTR_BUSPOWER,   attributes */
#endif
	USB_CFG_MAX_BUS_POWER / 2,	/* max USB current in 2mA units */

// B.3 AudioControl Interface Descriptors
// The AudioControl interface describes the device structure (audio function topology) 
// and is used to manipulate the Audio Controls. This device has no audio function 
// incorporated. However, the AudioControl interface is mandatory and therefore both 
// the standard AC interface descriptor and the classspecific AC interface descriptor 
// must be present. The class-specific AC interface descriptor only contains the header 
// descriptor.

// B.3.1 Standard AC Interface Descriptor
// The AudioControl interface has no dedicated endpoints associated with it. It uses the 
// default pipe (endpoint 0) for all communication purposes. Class-specific AudioControl 
// Requests are sent using the default pipe. There is no Status Interrupt endpoint provided.
	/* descriptor follows inline: */
	9,			/* sizeof(usbDescrInterface): length of descriptor in bytes */
	USBDESCR_INTERFACE,	/* descriptor type */
	0,			/* index of this interface */
	0,			/* alternate setting for this interface */
	0,			/* endpoints excl 0: number of endpoint descriptors to follow */
	1,			/* */
	1,			/* */
	0,			/* */
	0,			/* string index for interface */

// B.3.2 Class-specific AC Interface Descriptor
// The Class-specific AC interface descriptor is always headed by a Header descriptor 
// that contains general information about the AudioControl interface. It contains all 
// the pointers needed to describe the Audio Interface Collection, associated with the 
// described audio function. Only the Header descriptor is present in this device 
// because it does not contain any audio functionality as such.
	/* descriptor follows inline: */
	9,			/* sizeof(usbDescrCDC_HeaderFn): length of descriptor in bytes */
	36,			/* descriptor type */
	1,			/* header functional descriptor */
	0x0, 0x01,		/* bcdADC */
	9, 0,			/* wTotalLength */
	1,			/* */
	1,			/* */

// B.4 MIDIStreaming Interface Descriptors

// B.4.1 Standard MS Interface Descriptor
	/* descriptor follows inline: */
	9,			/* length of descriptor in bytes */
	USBDESCR_INTERFACE,	/* descriptor type */
	1,			/* index of this interface */
	0,			/* alternate setting for this interface */
	2,			/* endpoints excl 0: number of endpoint descriptors to follow */
	1,			/* AUDIO */
	3,			/* MS */
	0,			/* unused */
	0,			/* string index for interface */

// B.4.2 Class-specific MS Interface Descriptor
	/* descriptor follows inline: */
	7,			/* length of descriptor in bytes */
	36,			/* descriptor type */
	1,			/* header functional descriptor */
	0x0, 0x01,		/* bcdADC */
	65, 0,			/* wTotalLength */

// B.4.3 MIDI IN Jack Descriptor
	/* descriptor follows inline: */
	6,			/* bLength */
	36,			/* descriptor type */
	2,			/* MIDI_IN_JACK desc subtype */
	1,			/* EMBEDDED bJackType */
	1,			/* bJackID */
	0,			/* iJack */

	/* descriptor follows inline: */
	6,			/* bLength */
	36,			/* descriptor type */
	2,			/* MIDI_IN_JACK desc subtype */
	2,			/* EXTERNAL bJackType */
	2,			/* bJackID */
	0,			/* iJack */

//B.4.4 MIDI OUT Jack Descriptor
	/* descriptor follows inline: */
	9,			/* length of descriptor in bytes */
	36,			/* descriptor type */
	3,			/* MIDI_OUT_JACK descriptor */
	1,			/* EMBEDDED bJackType */
	3,			/* bJackID */
	1,			/* No of input pins */
	2,			/* BaSourceID */
	1,			/* BaSourcePin */
	0,			/* iJack */

	/* descriptor follows inline: */
	9,			/* bLength of descriptor in bytes */
	36,			/* bDescriptorType */
	3,			/* MIDI_OUT_JACK bDescriptorSubtype */
	2,			/* EXTERNAL bJackType */
	4,			/* bJackID */
	1,			/* bNrInputPins */
	1,			/* baSourceID (0) */
	1,			/* baSourcePin (0) */
	0,			/* iJack */


// B.5 Bulk OUT Endpoint Descriptors

//B.5.1 Standard Bulk OUT Endpoint Descriptor
	/* descriptor follows inline: */
	9,			/* bLenght */
	USBDESCR_ENDPOINT,	/* bDescriptorType = endpoint */
	0x1,			/* bEndpointAddress OUT endpoint number 1 */
	3,			/* bmAttributes: 2:Bulk, 3:Interrupt endpoint */
	8, 0,			/* wMaxPacketSize */
	10,			/* bIntervall in ms */
	0,			/* bRefresh */
	0,			/* bSyncAddress */

// B.5.2 Class-specific MS Bulk OUT Endpoint Descriptor
	/* descriptor follows inline: */
	5,			/* bLength of descriptor in bytes */
	37,			/* bDescriptorType */
	1,			/* bDescriptorSubtype */
	1,			/* bNumEmbMIDIJack  */
	1,			/* baAssocJackID (0) */


//B.6 Bulk IN Endpoint Descriptors

//B.6.1 Standard Bulk IN Endpoint Descriptor
	/* descriptor follows inline: */
	9,			/* bLenght */
	USBDESCR_ENDPOINT,	/* bDescriptorType = endpoint */
	0x81,			/* bEndpointAddress IN endpoint number 1 */
	3,			/* bmAttributes: 2: Bulk, 3: Interrupt endpoint */
	8, 0,			/* wMaxPacketSize */
	10,			/* bIntervall in ms */
	0,			/* bRefresh */
	0,			/* bSyncAddress */

// B.6.2 Class-specific MS Bulk IN Endpoint Descriptor
	/* descriptor follows inline: */
	5,			/* bLength of descriptor in bytes */
	37,			/* bDescriptorType */
	1,			/* bDescriptorSubtype */
	1,			/* bNumEmbMIDIJack (0) */
	3,			/* baAssocJackID (0) */
};


uchar usbFunctionDescriptor(usbRequest_t * rq) {
  if (rq->wValue.bytes[1] == USBDESCR_DEVICE) {
    usbMsgPtr = (uchar *) deviceDescrMIDI;
    return sizeof(deviceDescrMIDI);
  } else {		/* must be config descriptor */
    usbMsgPtr = (uchar *) configDescrMIDI;
    return sizeof(configDescrMIDI);
  }
}


static uchar sendEmptyFrame;

#define setPortBit0(port, bit) *(port) &= ~_BV(bit);
#define setPortBit1(port, bit) *(port) |= _BV(bit);

#define setPortBit(port, bit, val) { \
    if ((val) == 0) { setPortBit0((port), (bit)); } else	\
      { setPortBit1((port), (bit)); }				\
}
#define getPortBit(port, bit) ((*(port) & _BV(bit)) >> (bit))

#define flipPortBit(port, bit) (*(port) ^= _BV(bit))

#ifdef MONAKA_PSG
#define gioSetLED(val) setPortBit(&PORTC, 4, (val))
#else
#define gioSetLED(val) setPortBit(&PORTB, 5, (val))
#endif

/* ------------------------------------------------------------------------- */
/* ----------------------------- USB interface ----------------------------- */
/* ------------------------------------------------------------------------- */

uchar usbFunctionSetup(uchar data[8]) {
  usbRequest_t *rq = (void *) data;

  if ((rq->bmRequestType & USBRQ_TYPE_MASK) == USBRQ_TYPE_CLASS) {	/* class request type */

    /*  Prepare bulk-in endpoint to respond to early termination   */
    if ((rq->bmRequestType & USBRQ_DIR_MASK) ==
	USBRQ_DIR_HOST_TO_DEVICE)
      sendEmptyFrame = 1;
  }
  
  return 0xff;
}


/*---------------------------------------------------------------------------*/
/* usbFunctionRead                                                           */
/*---------------------------------------------------------------------------*/

uchar usbFunctionRead(uchar * data, uchar len) {
	data[0] = 0;
	data[1] = 0;
	data[2] = 0;
	data[3] = 0;
	data[4] = 0;
	data[5] = 0;
	data[6] = 0;

	return 7;
}


/*---------------------------------------------------------------------------*/
/* usbFunctionWrite                                                          */
/*---------------------------------------------------------------------------*/

uchar usbFunctionWrite(uchar * data, uchar len) {
  /* flipPortBit(&PORTB, 5);		// DEBUG LED */
  /* parseMidiMessage(data, len); */
  return 1;
}

/*---------------------------------------------------------------------------*/
/* usbFunctionWriteOut                                                       */
/*                                                                           */
/* this Function is called if a MIDI Out message (from PC) arrives.          */
/*                                                                           */
/*---------------------------------------------------------------------------*/

void usbFunctionWriteOut(uchar * data, uchar len) {
  /* flipPortBit(&PORTB, 5);		*/
  parseMidiMessage(data, len);
}

/*---------------------------------------------------------------------------*/
/* PSGのハンドラ */
/*---------------------------------------------------------------------------*/
uint16_t getTp(uchar toneNo) {
  return pgm_read_word(&freq_table[toneNo]);
}

#ifdef MONAKA_PSG
/************ PSG Shield PIN Assignment
 PSG : AVR
  /WR: PB3
  /CS: PB5  also this line have LED
  A0 : PB2
  /IC: PB4 (should always be H, also have SWITCH)
  D0 : PB1
  D1 : PB0
  D2 : PD7
  D3 : PD6
  D4 : PD5
  D5 : PD4
  D6 : PD3
  D7 : PD1 PATCHED (was PD1)

  A4 : LED (for monitoring)
*/

struct pinMappingTable_p {
  volatile uchar *port;
  uchar bit;
} pinMappingTable[] = {
  {&PORTB, 1},
  {&PORTB, 0},
  {&PORTD, 7},
  {&PORTD, 6},
  {&PORTD, 5},
  {&PORTD, 4},
  {&PORTD, 3},
  {&PORTD, 1}
};

#define SET_DATA(data) {						\
    uchar i; 								\
    for (i = 0 ; i < 8 ; i++ ) {				      	\
      if (((data >> i) &0x01) == 1) {				      	\
	setPortBit1(pinMappingTable[i].port, pinMappingTable[i].bit); 	\
      } else {								\
	setPortBit0(pinMappingTable[i].port, pinMappingTable[i].bit); 	\
      }									\
    }									\
  }
#define SET_A0_DATA() { setPortBit1(&PORTB, 2); }
#define SET_A0_ADDRESS()  { setPortBit0(&PORTB, 2); }
#define PSG_CS1(psgNo) setPortBit1(&PORTB, 5)
#define PSG_WE1(psgNo) setPortBit1(&PORTB, 3)
#define PSG_CS0(psgNo) setPortBit0(&PORTB, 5)
#define PSG_WE0(psgNo) { setPortBit0(&PORTB, 3); setPortBit0(&PORTB, 3); }

#else

/************ for original ginger board with 2 PSG
 PSG : AVR
  /WR & /CS (PSG0) : PC4
  /WR & /CS (PSG1) : PC5
  A0 : PD3
  /IC: (PULL-UP)
  D0 : PC0
  D1 : PC1
  D2 : PC2
  D3 : PC3
  D4 : PD4
  D5 : PD5
  D6 : PD6
  D7 : PD7

  PB5: LED (for monitoring)

*/
#define SET_DATA(data) {			\
    PORTC = ((PORTC & 0xf0) | ((data) & 0x0f));	\
    PORTD = ((PORTD & 0x0f) | ((data) & 0xf0));	\
  }
#define SET_A0_DATA() { setPortBit1(&PORTD, 3); }
#define SET_A0_ADDRESS()  { setPortBit0(&PORTD, 3); }
#define PSG_CS0(psgNo) ;
#define PSG_CS1(psgNo) ;
#define PSG_WE0(psgNo) {			\
    if ((psgNo) == 0) {		/* PC4 */ 	\
      setPortBit0(&PORTC, 4); 			\
    } else {			/* PC5 */ 	\
      setPortBit0(&PORTC, 5); 			\
    }\
}

#define PSG_WE1(psgNo) {			\
    if ((psgNo) == 0) {		/* PC4 */ 	\
      setPortBit1(&PORTC, 4); 			\
    } else {			/* PC5 */ 	\
      setPortBit1(&PORTC, 5); 			\
    }						\
  }

#endif

void setPSG(uchar psgNo, uchar regNo, uchar val) {

  PSG_CS0(psgNo);
  /* アドレスを設定する */
  SET_A0_ADDRESS();
  SET_DATA(regNo);
  PSG_WE0(psgNo);
  PSG_WE1(psgNo);

  PSG_CS1(psgNo);

  
  PSG_CS0(psgNo);
  /* データを設定する */
  SET_A0_DATA();
  SET_DATA(val);
  PSG_WE0(psgNo);
  PSG_WE1(psgNo);

  PSG_CS1(psgNo);
}

/* PSGの周波数をセットする */
#define setPSGFreq(psgNo, ch, tp) { \
  setPSG(psgNo, ch * 2, tp & 0xff); \
  setPSG(psgNo, (ch * 2) + 1, (tp & 0xff00) >>8);\
}

/* PSGのノイズ周波数をセットする */
#define setPSGNoiseFreq(psgNo, fn) setPSG((psgNo), 0x06, (fn) & 0b00011111)


/* PSGのミキサをセットする */
#define setPSGMixer(psgNo) { \
  setPSG(psgNo, 0x07,  0b00111000); /* 0を書き込むと出力される */ \
  /*    $07 = MIXER SETTING   */ \
  /*  D7 D6 D5 D4 D3 D2 D1 D0 */ \
  /*   X  X  C  B  A  C  B  A */ \
  /*         +NOISE+  +TONE+  */ \
}

/* PSGの音量をセットする */
#define setPSGVolume(psgNo, ch, val) setPSG((psgNo), (ch) + 0x08, (val))

/* PSGの周波数と音量をセットする*/
#define setPSGNoteVolume(psgNo, ch, note, val) { 	\
    setPSGVolume((psgNo), (ch), (val) & 0x0f);		\
    setPSGFreq((psgNo), (ch), getTp(note));		\
}

#define setPSGNoteOff(psgNo, ch) { \
    setPSG(psgNo, ch * 2, 0); \
    setPSG(psgNo, (ch * 2) + 1, 0); \
}

void setPSGVolumeAll(uchar psgNo) {
  uchar i;
  for (i = 0 ; i < 3 ; i++) {
    setPSGVolume(psgNo, i,  0xf);
  }
}

/* PSGを初期化する */
void initPSG(uchar psgNo) {
#ifdef MONAKA_PSG
  // setPortBit1(&PORTB,4) ; 	/* clear reset conditions */
  PSG_WE1(0);                   /* /WR=H */
  PSG_CS1(0);                   /* /CS=H */
#endif
  setPSGVolumeAll(psgNo);
  setPSGMixer(psgNo);
}

//-----------------------------------------------
/* note bufferの処理 */

void initNote() {
  uchar i;
  for (i = 0 ; i < MAX_NOTE ; i++) {
    notes[i].noteNo = NOTE_OFF;
    notes[i].volume = 0;
    notes[i].order = 0;
  }
  onCount = 0;
}

uchar findNote(uchar note) {
  uchar i;
  for (i = 0 ; i < MAX_NOTE ; i++) {
    if (note == notes[i].noteNo) {
      return i;
    }
  }
  return 0xff;
}

uchar findFreeSlot() {
  uchar i, rtn = 0;
  if (onCount < MAX_NOTE) {	/* スロット空きあり */
    for (i = 0 ; i < MAX_NOTE ; i++) {
      if (notes[i].order == 0) {
	return i;
      }
    }
  } else {		    /* スロットがいっぱいなので空きをつくる */
    for (i = 0 ; i < MAX_NOTE ; i++) {
      /* 発音番号を繰り下げながら最も発音が古いもの(order=1)を探す */
      if ((--notes[i].order) == 0) {
	rtn = i;
	/* いったん発音を止める？ */
	/* 	setPSGNoteOff(notes[i].psgNo, notes[i].ch); */
      }
    }
    onCount --;
    return rtn;
  }
  return 0xff;			/* LOGIC ERROR */
}

void allocNote(uchar note, uchar vol) {
  uchar i;
  if ((i = findNote(note)) == 0xff) { /* 同じ音階を発声中でない場合のみ */
    i= findFreeSlot();	/* 必ず成功する */
    notes[i].noteNo = note;
    notes[i].volume = vol;
    notes[i].order = (++ onCount);
    setPSGNoteVolume(notes[i].psgNo, notes[i].ch, note, vol);
  }
}

void delNote(uchar note) {
  uchar i;
  uchar j;
  if ((i = findNote(note)) != 0xff) {		/* 発音中の場合 */
    for (j = 0 ; j < MAX_NOTE ; j++) { /* 発音番号を繰り上げる */
      if ((notes[j].order > 0) && (notes[j].order > (notes[i].order))) {
	notes[j].order --;
      }
    }
    notes[i].noteNo = NOTE_OFF;
    notes[i].volume = 0;
    notes[i].order = 0;
    setPSGNoteOff(notes[i].psgNo, notes[i].ch);
    onCount --;
  }
}

void allNotesOff() {
  int i;
  initNote();
  for (i = 0 ; i < MAX_NOTE; i++) {
    setPSGNoteOff(notes[i].psgNo, notes[i].ch);
  }
}

/*----------------------------------------------- */
/* ノートON処理*/

#ifdef TEST
/* Simple test ... */
#define NoteON(note, vol) setPSGNoteVolume(0, 0, (note), (vol))
#else
#define NoteON(note, vol) allocNote((note), (vol))
#endif

/* ノートOFF処理 */

#ifdef TEST
#define NoteOFF(note) setPSGNoteOff(0, 0)
#else
#define NoteOFF(note) delNote((note))
#endif

/* ----------------------------------------------- */

void parseMidiMessage(uchar *data, uchar len) {
  uchar cin = (*data) & 0x0f;	/* CABLE NOを無視する */
  uchar Rch  = (*(data+1)) & 0x0f;
  uchar note = (*(data+2)) & 0x7f;
  uchar vol  = ((*(data+3)) >> 3) & 0x0f;

  if (Rch != 0)			/* only channel 0 */
    return;
  switch(cin) {
  case 8:			/* NOTE OFF */
    NoteOFF(note);
    gioSetLED(0);
    break;
  case 9:			/* NOTE ON */
    if( vol == 0){
      NoteOFF(note);
      gioSetLED(0);
    } else {
      NoteON(note, vol);
      gioSetLED(1);
    }
    break;
  case 14:			/* PITCH BEND */
    // PBBuf = (*(data + 3)) & 0x7f; /* use only MSB(7bit) */
    break;
    //  case 0:			/* MISC FUNCTION CODE */
    //  case 1:			/* CABLE EVENT */
    //  case 2:			/* SYSTEM COMMON (2 byte) */
    //  case 3:			/* SYSTEM COMMON (3 byte) */
    //  case 4:			/* SYS EX START */
    //  case 5:			/* SYSTEM EX end (with 1 byte) */
    //  case 6:			/* SYSTEM EX end (with 2 byte) */
    //  case 7:			/* SYSTEM EX end (with 3 byte) */
    //  case 10:		/* POLY KEY PRESS */
    //  case 11:		/* CONTROL CHANGE */
    //  case 12:		/* PROGRAM CHANGE */
    //  case 13:		/* CHANNEL PRESSURE */
    //  case 15:		/* SINGLE BYTE */
    //    break;
  }

  if (len > 4) {
    parseMidiMessage(data+4, len-4);
  }
}


/*---------------------------------------------------------------------------*/
/* hardwareInit                                                              */
/*---------------------------------------------------------------------------*/

static void hardwareInit(void) {
  uchar i, j;

  /* activate pull-ups except on USB lines */
  USB_CFG_IOPORT =
    (uchar) ~ ((1 << USB_CFG_DMINUS_BIT) |
	       (1 << USB_CFG_DPLUS_BIT));
  /* all pins input except USB (-> USB reset) */
#ifdef USB_CFG_PULLUP_IOPORT
  /* use usbDeviceConnect()/usbDeviceDisconnect() if available */
  USBDDR = 0;		/* we do RESET by deactivating pullup */
  usbDeviceDisconnect();
#else
  USBDDR = (1 << USB_CFG_DMINUS_BIT) | (1 << USB_CFG_DPLUS_BIT);
#endif

  j = 0;
  while (--j) {		/* USB Reset by device only required on Watchdog Reset */
    i = 0;
    while (--i);	/* delay >10ms for USB reset */
  }
#ifdef USB_CFG_PULLUP_IOPORT
  usbDeviceConnect();
#else
  USBDDR = 0;		/*  remove USB reset condition */
#endif
}

void portInit(void) {
  /* ポートの初期化をする */
#ifdef MONAKA_PSG
  DDRB |=  0b00101111;		/* OUTPUT */
  DDRB &=~ 0b00010000;		/* INPUT */
  DDRC |=  0b00010000;		/* OUTPUT */
  DDRD |=  0b11111010;		/* OUTPUT */

  PORTB &= 0b00010000;		/* PB4 shoule be input with pull-up */
  PORTC  = 0b00010000;		/* LED ON */

#else
  DDRB |=  0b00100000;		/* OUTPUT */
  DDRC |=  0b00111111;		/* OUTPUT */
  DDRD |=  0b11111000;		/* OUTPUT */

  PORTC =  0b00110000;		/* CS-A, CS-B disable */
#endif

}

int main(void)
{
  wdt_enable(WDTO_1S);
  hardwareInit();
  odDebugInit();
  portInit();
  usbInit();

  initPSG(0);			/* initialize PSG0 */
#if (N_PSG == 2)
  initPSG(1);			/* initialize PSG1 */
#endif
  sendEmptyFrame = 0;

  sei();

  for (;;) {			/* main event loop */
    wdt_reset();
    usbPoll();

    /* USB送信処理 */
    if (usbInterruptIsReady()) {
      /* send midi message here */
      /* 何もしない */
    }	/* usbInterruptIsReady() */

  } /* main event loop */
  return 0;
}
